@@ -1,30 +1,40 @@
 from Globals import InitializeClass
-from OFS.SimpleItem import SimpleItem
+from persistent import Persistent
+from Acquisition import Explicit
 from zope.app import zapi
-from zope.app.intid import IntIds
+from zope.app.intid import IntIds as z3IntIds
 from zope.app.intid.interfaces import IIntIds
 from zope.app.intid.interfaces import IntIdAddedEvent, IntIdRemovedEvent
+from zope.app.container.interfaces import IObjectAddedEvent, IObjectRemovedEvent
 from zope.app.keyreference.interfaces import IKeyReference, NotYet
 from zope.event import notify
 from zope.interface import implements
 from five.intid.interfaces import UnsettableAttributeError
+try:
+    from Products.CMFCore.interfaces import IDirectoryView
+except ImportError:
+    from zope.interface import Interface
+    class IDirectoryView(Interface):
+        pass
 
-class OFSIntIds(SimpleItem, IntIds):
+_marker = []
+
+class IntIds(z3IntIds):
     """ zope2ish intid utility """
     implements(IIntIds)
 
     meta_type="IntId Utility"
 
     def __init__(self, id_=IIntIds.__name__):
-        self.id = id_
-        super(OFSIntIds, self).__init__()
+        self.id = self.__name__ = id_
+        super(IntIds, self).__init__()
+
+    def getId(self, ob=_marker):
+        # Compatibility with SimpleItem
+        if ob is _marker:
+            return self.__name__
+        return z3IntIds.getId(self, ob)
 
-    def getId(*args):
-        # sweet compatibility
-        if len(args) == 1:
-            return SimpleItem.getId(args[0])
-        return IntIds.getId(*args)
-    
     def register(self, ob):
         key = IKeyReference(ob)
         if key in self.ids:
@@ -43,6 +53,16 @@ class OFSIntIds(SimpleItem, IntIds):
         del self.refs[uid]
         del self.ids[key]
 
+InitializeClass(IntIds)
+
+class OFSIntIds(IntIds, Explicit):
+    """Mixin acquisition for non-lsm sites"""
+
+    def manage_fixupOwnershipAfterAdd(self):
+        pass
+    def wl_isLocked(self):
+        return False
+
 InitializeClass(OFSIntIds)
 
 # @@ these are "sloppy" subscribers that let objects that have not
@@ -53,17 +73,19 @@ def addIntIdSubscriber(ob, event):
     Registers the object added in all unique id utilities and fires
     an event for the catalogs.
     """
-
+    # Ignore CMF's IDirectoryView stuff as this shouldn't be persisted
+    if IDirectoryView.providedBy(ob):
+        return
     utilities = tuple(zapi.getAllUtilitiesRegisteredFor(IIntIds))
     if utilities: # assert that there are any utilites
         key = None
         try:
             key = IKeyReference(ob, None)
-        except NotYet:  
+        except NotYet:
             pass
         except UnsettableAttributeError:
             pass
-            
+
         # Register only objects that adapt to key reference
         if key is not None:
             for utility in utilities:
@@ -77,7 +99,8 @@ def removeIntIdSubscriber(ob, event):
     Removes the unique ids registered for the object in all the unique
     id utilities.
     """
-
+    if IDirectoryView.providedBy(ob):
+        return
     utilities = tuple(zapi.getAllUtilitiesRegisteredFor(IIntIds))
     if utilities:
         key = None
@@ -87,7 +110,7 @@ def removeIntIdSubscriber(ob, event):
             pass
         except UnsettableAttributeError:
             pass
-            
+
         # Register only objects that adapt to key reference
         if key is not None:
             # Notify the catalogs that this object is about to be removed.
@@ -97,3 +120,35 @@ def removeIntIdSubscriber(ob, event):
                     utility.unregister(key)
                 except KeyError:
                     pass
+
+
+def moveIntIdSubscriber(ob, event):
+    """A subscriber to ObjectMovedEvent
+
+    Updates the stored path for the object in all the unique
+    id utilities.
+    """
+    if IDirectoryView.providedBy(ob):
+        return
+    if IObjectRemovedEvent.providedBy(event) or \
+           IObjectAddedEvent.providedBy(event):
+        return
+    utilities = tuple(zapi.getAllUtilitiesRegisteredFor(IIntIds))
+    if utilities:
+        key = None
+        try:
+            key = IKeyReference(ob, None)
+        except NotYet: # @@ temporary fix
+            pass
+        except UnsettableAttributeError:
+            pass
+
+        # Update objects that adapt to key reference
+        if key is not None:
+            for utility in utilities:
+                try:
+                    uid = utility.getId(ob)
+                    utility.refs[uid] = key
+                    utility.ids[key] = uid
+                except KeyError:
+                    pass
